@@ -3,7 +3,7 @@ package Promise::ES6;
 use strict;
 use warnings;
 
-our $VERSION = '0.24_02';
+our $VERSION = '0.24_03';
 
 =encoding utf-8
 
@@ -41,6 +41,12 @@ Promise::ES6 - ES6-style promises in Perl
 
 =head1 DESCRIPTION
 
+=begin html
+
+<a href='https://coveralls.io/github/FGasper/p5-Promise-ES6?branch=master'><img src='https://coveralls.io/repos/github/FGasper/p5-Promise-ES6/badge.svg?branch=master' alt='Coverage Status' /></a>
+
+=end html
+
 This module provides a Perl implementation of L<promises|https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Using_promises>, a useful pattern
 for coordinating asynchronous tasks.
 
@@ -670,6 +676,9 @@ sub _do_carp {
 # inside the promise. We’ll clear that self-reference once the promise
 # is finished, which avoids memory leaks.
 #
+# (F::AA behaves as it does because Future.pm retains “back-references”
+# rather than the “forward references” we use here.)
+#
 sub _immortalize {
     my $method = $_[0];
 
@@ -690,18 +699,16 @@ sub AWAIT_CLONE {
     _immortalize('new', ref $_[0], \&_noop);
 }
 
-sub AWAIT_DONE {
-    &_resolve;
-}
-sub AWAIT_FAIL {
-    &_reject;
-}
+*AWAIT_DONE = *_resolve;
+*AWAIT_FAIL = *_reject;
 
 sub AWAIT_IS_READY {
     exists $_[0]->{'_result'};
 }
 
 sub AWAIT_GET {
+    $_[0]{'handled'} = 1;
+
     return $_[0]{'_result'} if $_[0]{'_success'};
 
     die $_[0]{'_result'};
