@@ -7,7 +7,7 @@ use autodie;
 use Test::More;
 use Test::FailWarnings -allow_deps => 1;
 
-use constant _TEST_COUNT => 6;
+use constant _TEST_COUNT => 8;
 
 sub _FULL_BACKEND {
     return "Promise::ES6::" . $_[0]->_BACKEND();
@@ -21,8 +21,7 @@ sub run {
   SKIP: {
         eval { $class->_REQUIRE(); 1 } or skip "$class: Backend isn’t available: $@", _TEST_COUNT();
 
-        my $backend = $class->_BACKEND();
-        require "Promise/ES6/$backend.pm";
+        $class->_REQUIRE_BACKEND();
 
         $class->_test_normal();
         $class->_test_die_in_constructor();
@@ -31,9 +30,18 @@ sub run {
 
         $class->_test_call_again_in_callback();
         $class->_test_die_in_then();
+        $class->_test_die_in_catch();
+        $class->_test_finally_after_rejection_rejects();
     }
 }
 
+sub _REQUIRE_BACKEND {
+    my ($class) = @_;
+
+    my $backend = $class->_BACKEND();
+    require "Promise/ES6/$backend.pm";
+}
+
 sub _test_call_again_in_callback {
     my ($class) = @_;
 
@@ -119,11 +127,11 @@ sub _test_reject {
 
     push @things, 'c';
 
-    $promise->catch( sub { push @things, shift } );
+    my $p2 = $promise->catch( sub { push @things, shift } );
 
     push @things, 'e';
 
-    $class->_RESOLVE($promise);
+    $class->_RESOLVE($p2);
 
     push @things, 'f';
 
@@ -147,7 +155,7 @@ sub _test_die_in_constructor {
 
     push @things, 'c';
 
-    $promise->catch( sub { push @things, shift } );
+    $promise = $promise->catch( sub { push @things, shift } );
 
     push @things, 'e';
 
@@ -167,17 +175,17 @@ sub _test_die_in_then {
 
     my @things;
 
-    my $promise = $class->_FULL_BACKEND()->resolve(123)->then( sub {
+    my $promise = $class->_FULL_BACKEND()->resolve(1)->then( sub {
         die "123\n";
     } );
 
     push @things, 'c';
 
-    $promise->catch( sub { push @things, shift } );
+    my $p2 = $promise->catch( sub { push @things, shift } );
 
     push @things, 'e';
 
-    $class->_RESOLVE($promise);
+    $class->_RESOLVE($p2);
 
     push @things, 'f';
 
@@ -188,4 +196,48 @@ sub _test_die_in_then {
     );
 }
 
+sub _test_die_in_catch {
+    my ($class) = @_;
+
+    my @things;
+
+    my $promise = $class->_FULL_BACKEND()->reject(1)->catch( sub {
+        die "123\n";
+    } );
+
+    push @things, 'c';
+
+    my $p2 = $promise->catch( sub { push @things, shift } );
+
+    push @things, 'e';
+
+    $class->_RESOLVE($p2);
+
+    push @things, 'f';
+
+    is(
+        "@things",
+        "c e 123\n f",
+        'catch() callback invoked asynchronously',
+    );
+}
+
+sub _test_finally_after_rejection_rejects {
+    my ($class) = @_;
+
+    my $caught;
+
+    {
+        $class->_RESOLVE(
+            $class->_FULL_BACKEND()->reject(6666)->finally( sub {} )->catch( sub { $caught = shift } )
+        );
+    }
+
+    like(
+        $caught,
+        qr<6666>,
+        'finally() rejects ',
+    );
+}
+
 1;
